# -*- coding: utf-8 -*-
from __future__ import unicode_literals

from django.db import models, migrations
import django.db.models.deletion
from django.conf import settings


class Migration(migrations.Migration):

    dependencies = [
        migrations.swappable_dependency(settings.AUTH_USER_MODEL),
        ('contenttypes', '0001_initial'),
    ]

    operations = [
        migrations.CreateModel(
            name='Revision',
            fields=[
                ('id', models.AutoField(verbose_name='ID', serialize=False, auto_created=True, primary_key=True)),
                ('manager_slug', models.CharField(default='default', max_length=200, db_index=True)),
                ('date_created', models.DateTimeField(auto_now_add=True, help_text='The date and time this revision was created.', verbose_name='date created', db_index=True)),
                ('comment', models.TextField(help_text='A text comment on this revision.', verbose_name='comment', blank=True)),
                ('user', models.ForeignKey(on_delete=django.db.models.deletion.SET_NULL, blank=True, to=settings.AUTH_USER_MODEL, help_text='The user who created this revision.', null=True, verbose_name='user')),
            ],
            options={
                "ordering": ("-pk",)
            },
            bases=(models.Model,),
        ),
        migrations.CreateModel(
            name='Version',
            fields=[
                ('id', models.AutoField(verbose_name='ID', serialize=False, auto_created=True, primary_key=True)),
                ('object_id', models.TextField(help_text='Primary key of the model under version control.')),
                ('object_id_int', models.IntegerField(help_text="An indexed, integer version of the stored model's primary key, used for faster lookups.", null=True, db_index=True, blank=True)),
                ('format', models.CharField(help_text='The serialization format used by this model.', max_length=255)),
                ('serialized_data', models.TextField(help_text='The serialized form of this version of the model.')),
                ('object_repr', models.TextField(help_text='A string representation of the object.')),
                ('content_type', models.ForeignKey(help_text='Content type of the model under version control.', on_delete=django.db.models.deletion.CASCADE, to='contenttypes.ContentType')),
                ('revision', models.ForeignKey(help_text='The revision that contains this version.', on_delete=django.db.models.deletion.CASCADE, to='reversion.Revision')),
            ],
            options={
                "ordering": ("-pk",)
            },
            bases=(models.Model,),
        ),
    ]



from django.conf.urls import url, include
from django.contrib import admin

admin.autodiscover()

urlpatterns = [

    url(r"^admin/", admin.site.urls),

    url(r"^test-app/", include("test_app.urls")),

]



# -*- coding: utf-8 -*-
# Generated by Django 1.9.6 on 2016-06-01 16:00
from __future__ import unicode_literals

from collections import defaultdict
from django.db import DEFAULT_DB_ALIAS, migrations, models, router
from django.apps import apps as live_apps


def de_dupe_version_table(apps, schema_editor):
    """
    Removes some duplicate Version models that may have crept into the database and will prevent the
    unique index being added by migration 0004.
    """
    db_alias = schema_editor.connection.alias
    Version = apps.get_model("reversion", "Version")
    keep_version_ids = Version.objects.using(db_alias).order_by().values_list(
        # Group by the unique constraint we intend to enforce.
        "revision_id",
        "content_type_id",
        "object_id",
    ).annotate(
        # Add in the most recent id for each duplicate row.
        max_pk=models.Max("pk"),
    ).values_list("max_pk", flat=True)
    # Do not do anything if we're keeping all ids anyway.
    if keep_version_ids.count() == Version.objects.using(db_alias).all().count():
        return
    # Delete all duplicate versions. Can't do this as a delete with subquery because MySQL doesn't like running a
    # subquery on the table being updated/deleted.
    delete_version_ids = list(Version.objects.using(db_alias).exclude(
        pk__in=keep_version_ids,
    ).values_list("pk", flat=True))
    Version.objects.using(db_alias).filter(
        pk__in=delete_version_ids,
    ).delete()


def set_version_db(apps, schema_editor):
    """
    Updates the db field in all Version models to point to the correct write
    db for the model.
    """
    db_alias = schema_editor.connection.alias
    Version = apps.get_model("reversion", "Version")
    content_types = Version.objects.using(db_alias).order_by().values_list(
        "content_type_id",
        "content_type__app_label",
        "content_type__model"
    ).distinct()
    model_dbs = defaultdict(list)
    for content_type_id, app_label, model_name in content_types:
        # We need to be able to access all models in the project, and we can't
        # specify them up-front in the migration dependencies. So we have to
        # just get the live model. This should be fine, since we don't actually
        # manipulate the live model in any way.
        try:
            model = live_apps.get_model(app_label, model_name)
        except LookupError:
            # If the model appears not to exist, play it safe and use the default db.
            db = "default"
        else:
            db = router.db_for_write(model)
        model_dbs[db].append(content_type_id)
    # Update db field.
    # speedup for case when there is only default db
    if DEFAULT_DB_ALIAS in model_dbs and len(model_dbs) == 1:
        Version.objects.using(db_alias).update(db=DEFAULT_DB_ALIAS)
    else:
        for db, content_type_ids in model_dbs.items():
            Version.objects.using(db_alias).filter(
                content_type__in=content_type_ids
            ).update(db=db)


class Migration(migrations.Migration):

    dependencies = [
        ('reversion', '0002_auto_20141216_1509'),
    ]

    operations = [
        migrations.RemoveField(
            model_name='revision',
            name='manager_slug',
        ),
        migrations.RemoveField(
            model_name='version',
            name='object_id_int',
        ),
        migrations.AlterField(
            model_name='version',
            name='object_id',
            field=models.CharField(help_text='Primary key of the model under version control.', max_length=191),
        ),
        migrations.AlterField(
            model_name='revision',
            name='date_created',
            field=models.DateTimeField(db_index=True, help_text='The date and time this revision was created.', verbose_name='date created'),
        ),
        migrations.AddField(
            model_name='version',
            name='db',
            field=models.CharField(null=True, help_text='The database the model under version control is stored in.', max_length=191),
        ),
        migrations.RunPython(de_dupe_version_table),
        migrations.RunPython(set_version_db),
    ]



# -*- coding: utf-8 -*-
# Generated by Django 1.9.7 on 2016-06-11 12:02
from __future__ import unicode_literals

from django.db import migrations, models


class Migration(migrations.Migration):

    dependencies = [
        ('reversion', '0003_auto_20160601_1600'),
    ]

    operations = [
        migrations.AlterField(
            model_name='version',
            name='db',
            field=models.CharField(help_text='The database the model under version control is stored in.', max_length=191),
        ),
        migrations.AlterUniqueTogether(
            name='version',
            unique_together=set([('db', 'content_type', 'object_id', 'revision')]),
        ),
    ]



import django


def remote_field(field):
    # remote_field is new in Django 1.9
    return field.remote_field if hasattr(field, 'remote_field') else field.rel


def remote_model(field):
    # remote_field is new in Django 1.9
    return field.remote_field.model if hasattr(field, 'remote_field') else field.rel.to


def is_authenticated(user):
    if django.VERSION < (1, 10):
        return user.is_authenticated()
    return user.is_authenticated



import sys
from reversion.revisions import create_revision as create_revision_base
from reversion.views import _request_creates_revision, _set_user_from_request, create_revision


class RevisionMiddleware(object):

    """Wraps the entire request in a revision."""

    manage_manually = False

    using = None

    atomic = True

    def __init__(self, get_response=None):
        super(RevisionMiddleware, self).__init__()
        # Support Django 1.10 middleware.
        if get_response is not None:
            self.get_response = create_revision(
                manage_manually=self.manage_manually,
                using=self.using,
                atomic=self.atomic
            )(get_response)

    def process_request(self, request):
        if _request_creates_revision(request):
            context = create_revision_base(
                manage_manually=self.manage_manually,
                using=self.using,
                atomic=self.atomic
            )
            context.__enter__()
            if not hasattr(request, "_revision_middleware"):
                setattr(request, "_revision_middleware", {})
            request._revision_middleware[self] = context

    def _close_revision(self, request, is_exception):
        if self in getattr(request, "_revision_middleware", {}):
            _set_user_from_request(request)
            request._revision_middleware.pop(self).__exit__(*sys.exc_info() if is_exception else (None, None, None))

    def process_response(self, request, response):
        self._close_revision(request, False)
        return response

    def process_exception(self, request, exception):
        self._close_revision(request, True)

    def __call__(self, request):
        return self.get_response(request)
